@@ -2,18 +2,20 @@
 # coding: utf8
 # Part of the bioread package for reading BIOPAC data.
 #
-# Copyright (c) 2023 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2025 Board of Regents of the University of Wisconsin System
 #
 # Written Nate Vack <njvack@wisc.edu> with research from John Ollinger
 # at the Waisman Laboratory for Brain Imaging and Behavior, University of
 # Wisconsin-Madison
-# Project home: http://github.com/njvack/bioread
 
 # This contains the entry point for an executable to convert BIOPAC
 # AcqKnowledge files into Matlab files.
 
 """Convert an AcqKnowledge file to a MATLAB file.
 
+This program is deprecated -- MATLAB can read HDF5 files natively, so I
+highly recommend using acq2hdf5 instead.
+
 Usage:
   acq2mat [options] <acq_file> <mat_file>
   acq2mat -h | --help
@@ -21,14 +23,14 @@ Usage:
 
 Options:
   -c, --compress  Save compressed Matlab file
-  --data-only     Only save data and required header information -- do not
-                  save event markers.
+  -s, --single    Save data in single precision format
+  --data-only     Only save data and required metadata -- do not
+                  save event markers, journal, or most header information
+  -h, --help      Show this help message and exit
 
 Note: scipy is required for this program.
 """
 
-from __future__ import absolute_import
-
 import sys
 from docopt import docopt
 
@@ -45,7 +47,7 @@ def main(argv=None):
     amr.run()
 
 
-class AcqToMatRunner(object):
+class AcqToMatRunner:
     """The little wrapper class that converts acq files to mat files"""
 
     def __init__(self, argv, err=None):
@@ -57,28 +59,26 @@ class AcqToMatRunner(object):
     def run(self):
         old_err = sys.stderr
         sys.stderr = self.err
-        pargs = docopt(
-            __doc__,
-            self.argv,
-            version=meta.version_description)
+        pargs = docopt(__doc__, self.argv, version=meta.version_description)
         try:
             import scipy  # noqa -- catch this error before matlabwriter
-        except Exception:
+        except ImportError:
             sys.stderr.write("scipy is required for writing matlab files\n")
             sys.exit(1)
-        infile = pargs['<acq_file>']
-        matfile = pargs['<mat_file>']
-        compress = pargs['--compress']
+        infile = pargs["<acq_file>"]
+        matfile = pargs["<mat_file>"]
+        compress = pargs["--compress"]
+        single = pargs["--single"]
+        data_only = pargs["--data-only"]
 
         data = Reader.read(infile).datafile
+
         MatlabWriter.write_file(
-            data,
-            matfile,
-            compress=compress,
-            data_only=pargs['--data-only'])
+            data, matfile, compress=compress, data_only=data_only, single=single
+        )
 
         sys.stderr = old_err
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
