@@ -2,12 +2,11 @@
 # coding: utf8
 # Part of the bioread package for reading BIOPAC data.
 #
-# Copyright (c) 2023 Board of Regents of the University of Wisconsin System
+# Copyright (c) 2025 Board of Regents of the University of Wisconsin System
 #
 # Written Nate Vack <njvack@wisc.edu> with research from John Ollinger
 # at the Waisman Laboratory for Brain Imaging and Behavior, University of
 # Wisconsin-Madison
-# Project home: http://github.com/njvack/bioread
 
 # This contains the entry point for an executable to convert BIOPAC
 # AcqKnowledge files into text files
@@ -49,30 +48,26 @@ def main(argv=None):
     amr.run()
 
 
-class AcqToTxtRunner(object):
+class AcqToTxtRunner:
     """The little wrapper class that converts acq files to text files"""
 
     def __init__(self, argv):
         self.argv = argv
 
     def run(self):
-        pargs = docopt(
-            __doc__,
-            self.argv,
-            version=meta.version_description)
-        infile = pargs['<acq_file>']
+        pargs = docopt(__doc__, self.argv, version=meta.version_description)
+        infile = pargs["<acq_file>"]
         channel_indexes = None
-        if pargs['--channel-indexes']:
-            channel_indexes = [
-                int(i) for i in pargs['--channel-indexes'].split(',')]
+        if pargs["--channel-indexes"]:
+            channel_indexes = [int(i) for i in pargs["--channel-indexes"].split(",")]
         data = bioread.read(infile, channel_indexes=channel_indexes)
-        mval = pargs['--missing-as']
-        if pargs['--outfile']:
-            with open(pargs['--outfile'], 'w') as f:
+        mval = pargs["--missing-as"]
+        if pargs["--outfile"]:
+            with open(pargs["--outfile"], "w") as f:
                 txtwriter.write_text(data, f, channel_indexes, mval)
         else:
             txtwriter.write_text(data, sys.stdout, channel_indexes, mval)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
